package com.gic.download.utils;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.ToolUtil;
import com.gic.download.qo.QrcodeContent;
import com.gic.download.qo.QrcodeQO;
import com.gic.enterprise.context.RequestContext;
import com.gic.enterprise.dto.DownloadReportDTO;
import com.gic.enterprise.dto.QrcodeDTO;
import com.gic.enterprise.service.DownloadReportApiService;
import com.gic.enterprise.service.QrCodeApiService;
import com.gic.enterprise.utils.UserDetail;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.thirdparty.BucketNameEnum;
import com.gic.thirdparty.FileOperateUtils;
import com.gic.wechat.business.api.service.fwh.WeixinBaseFunService;
import com.gic.wechat.business.api.service.wxa.WeixinWxaFunService;
import com.google.zxing.BarcodeFormat;
import com.google.zxing.EncodeHintType;
import com.google.zxing.MultiFormatWriter;
import com.google.zxing.WriterException;
import com.google.zxing.common.BitMatrix;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;
/**
 * 二维码下载
 * @ClassName: QrcodeDownload 
 * @Description:  
 * @author guojuxing 
 * @date 2020/9/15 10:45 AM 
 */
public abstract class QrcodeDownload {
    private static final Logger log = LogManager.getLogger(QrcodeDownload.class);
    private static final int WIDTH = 1280;
    private static final int HEIGHT = 1280;
    /**
     * 默认是黑色
     */
    private static final int QRCOLOR = 0xFF000000;
    /**
     * 背景颜色
     */
    private static final int BGWHITE = 0xFFFFFFFF;
    private static final int PAGE_SIZE = 1000;

    private QrcodeQO qrcodeQO = null;
    private Integer total;
    private int qrcodeType;
    private DownloadReportApiService downloadReportApiService;
    private WeixinBaseFunService weixinBaseFunService;
    private WeixinWxaFunService weixinWxaFunService;
    private QrCodeApiService qrCodeApiService;

    public void takeQrcode(QrcodeQO qrcodeQO, int total, int qrcodeType,
                              DownloadReportApiService downloadReportApiService,
                              WeixinBaseFunService weixinBaseFunService,
                              WeixinWxaFunService weixinWxaFunService,
                              QrCodeApiService qrCodeApiService){
        this.qrcodeQO = qrcodeQO;
        this.downloadReportApiService = downloadReportApiService;
        this.weixinBaseFunService = weixinBaseFunService;
        this.weixinWxaFunService = weixinWxaFunService;
        this.qrCodeApiService = qrCodeApiService;
        this.total = total;
        this.qrcodeType = qrcodeType;
        Integer reportId = this.saveDownloadLog();
        String fileId = ToolUtil.randomUUID();
        String filePath = RequestContext.getContext().getRequest().getSession().getServletContext().getRealPath("/qrcode");
        filePath += File.separator + UserDetailUtils.getUserDetail().getEnterpriseId() + File.separator + fileId+File.separator+qrcodeQO.getFileName()+".zip";
        takeQrcodeTask(filePath, reportId, qrcodeQO.getFileName(), UserDetailUtils.getUserDetail());
    }

    private void takeQrcodeTask(final String filePath, final Integer reportId, final String fileName,final UserDetail userDetail){
        ExecutorPoolSingleton.getInstance().executeTask(new Runnable() {
            @Override
            public void run() {
                try {
                    File file = new File(filePath);
                    if(!file.exists()){
                        file.getParentFile().mkdirs();
                        file.createNewFile();
                    }
                    ZipOutputStream zipOutputStream = new ZipOutputStream(new FileOutputStream(file));
                    int pages = total%PAGE_SIZE > 0 ? total/PAGE_SIZE+1 : total/PAGE_SIZE;
                    for(int i = 1; i<= pages; i++){
                        List<QrcodeContent> contentList = getContent(i, PAGE_SIZE);
                        if(CollectionUtils.isEmpty(contentList)){
                            break;
                        }
                        for(QrcodeContent content : contentList){
                            QrcodeDTO dto = EntityUtil.changeEntityByJSON(QrcodeDTO.class, qrcodeQO);
                            dto.setEnterpriseId(userDetail.getEnterpriseId());
                            dto.setCustomParams(content.getCustomParams());
                            dto.setContent(content.getContent());
                            dto.setFileId(ToolUtil.randomUUID());
                            dto.setQrCodeType(qrcodeType);
                            ServiceResponse<Integer> response = qrCodeApiService.saveQrcodeDownload(dto);
                            if (response.isSuccess()) {
                                Integer qrcodeId = response.getResult();
                                if (qrcodeQO.getH5() == 1) {
                                    byte[] h5Qrcode = createH5Qrcode(content.getContent(), content.getTitle(), userDetail);
                                    ZipEntry e = new ZipEntry(content.getTitle() + File.separator + content.getTitle() + ".jpg");
                                    zipOutputStream.putNextEntry(e);
                                    zipOutputStream.write(h5Qrcode);
                                    zipOutputStream.flush();
                                }
                                if (StringUtils.isNotBlank(dto.getFwhIds())) {
                                    String[] fwhArr = dto.getFwhIds().split(",");
                                    for (String s : fwhArr) {
                                        byte[] fwhQrcode = createFwhQrcode(qrcodeId, dto.getType(), s, dto.getEndTime(), content.getTitle(), userDetail);
                                        if(fwhQrcode != null){
                                            ZipEntry e = new ZipEntry(content.getTitle() + File.separator + s + File.separator + content.getTitle() + ".jpg");
                                            zipOutputStream.putNextEntry(e);
                                            zipOutputStream.write(fwhQrcode);
                                            zipOutputStream.flush();
                                        }
                                    }
                                }
                                if (StringUtils.isNotBlank(dto.getAppletIds())) {
                                    String appletPath = content.getContent();
                                    if(StringUtils.isBlank(appletPath)){
                                        appletPath = "pages/index/index";
                                    }
                                    appletPath += "?qrcodeParam="+qrcodeId;
                                    String[] appletArr = dto.getAppletIds().split(",");
                                    for (String s : appletArr) {
                                        byte[] appletQrcode = createAppletQrcode(s, appletPath);
                                        if(appletQrcode != null){
                                            ZipEntry e = new ZipEntry(content.getTitle() + File.separator + s + File.separator + content.getTitle() + ".jpg");
                                            zipOutputStream.putNextEntry(e);
                                            zipOutputStream.write(appletQrcode);
                                            zipOutputStream.flush();
                                        }
                                    }
                                }
                            }
                        }
                    }
                    zipOutputStream.flush();
                    zipOutputStream.closeEntry();
                    zipOutputStream.close();
                    String url = FileOperateUtils.simpleUploadFileFromLocal(file, fileName+".zip", BucketNameEnum.COMPRESS_60000.getName());
                    downloadReportApiService.updateDownloadUrl(reportId, "https://"+url);
                }catch (Exception e) {
                    log.warn("下载二维码异常：", e);
                }
            }
        });
    }

    private Integer saveDownloadLog(){
        DownloadReportDTO reportDTO = new DownloadReportDTO();
        reportDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        reportDTO.setApplyUserId(UserDetailUtils.getUserDetail().getUserId());
        reportDTO.setReportStartTime(null);
        reportDTO.setReportEndTime(null);
        reportDTO.setFileName(qrcodeQO.getFileName());
        reportDTO.setDataContent("二维码");
        reportDTO.setDataUrl("-");
        reportDTO.setDataType(3);
        reportDTO.setDataCount(total);
        ServiceResponse<Integer> response = downloadReportApiService.saveDownloadReport(reportDTO);
        if(response.isSuccess() && response.getResult() != null){
            return response.getResult();
        }
        return 0;
    }

    private byte[] createFwhQrcode(Integer qrCodeId, int type, String appkey, Date endTime, String title, UserDetail userDetail){
        ServiceResponse<String> qRcodeUrl = null;
        if(type == 1){
            int expireSecond = Integer.valueOf((endTime.getTime() - System.currentTimeMillis())/1000+"");
            qRcodeUrl = this.weixinBaseFunService.getTemporaryQRcodeURL(appkey, qrCodeId + "", expireSecond);
        }else {
            qRcodeUrl = this.weixinBaseFunService.getQRcodeURL(appkey, qrCodeId + "");
        }
        if(qRcodeUrl.isSuccess()){
            return createQrcode(qRcodeUrl.getResult(), userDetail.getEnterpriseInfo().getLogo(), title);
        }
        return null;
    }

    private byte[] createAppletQrcode(String appkey, String path){
        ServiceResponse<Map<String, Object>> wxaQrcode = this.weixinWxaFunService.createWxaQrcode(appkey, path, 1280);
        if(wxaQrcode.isSuccess()){
            Map<String, Object> result = wxaQrcode.getResult();
            return (byte[]) result.get("content");
        }
        return null;
    }
    private byte[] createH5Qrcode(String content, String title, UserDetail userDetail){
        return createQrcode(content, userDetail.getEnterpriseInfo().getLogo(), title);
    }

    /**
     * 获取数据
     * @Title: getContent 
     * @Description:
     * @author guojuxing
     * @param pageSize
     * @param pageNum 
     * @return  java.util.List<com.gic.download.qo.QrcodeContent> 
     */
    protected abstract List<QrcodeContent> getContent(int pageSize, int pageNum);

    private byte[] createQrcode(String url, String logoPic, String logoName) {
        byte[] imageByte = null;
        // 图像类型
        String format = "png";
        Map<EncodeHintType, Object> hints = new HashMap<EncodeHintType, Object>();
        hints.put(EncodeHintType.CHARACTER_SET, "UTF-8");
        try {
            BufferedImage bim = getQrCodeBufferedImage(url, BarcodeFormat.QR_CODE, WIDTH, HEIGHT, hints);
            imageByte = addLogoQrCode(bim, getPicByte(logoPic), logoName, format);
            return imageByte;
        } catch (Exception e) {
            log.error("创建二维码失败", e);
            return imageByte;
        }
    }

    private byte[] getPicByte(String imgUrl) {
        if(StringUtils.isBlank(imgUrl)){
            return null;
        }
        HttpURLConnection conn = null;
        try {
            URL url = new URL(imgUrl);
            try {
                conn = (HttpURLConnection) url.openConnection();
                conn.setRequestMethod("GET");
                InputStream in = conn.getInputStream();
                return IOUtils.toByteArray(in);
            } catch (IOException e) {
                log.warn(e);
                return null;
            }
        } catch (MalformedURLException e) {
            log.warn(e);
            return null;
        }
    }

    /**
     * 生成LOGO二维码图片
     *
     * @param bim
     *            图片对象
     * @param logoPic
     *            logo图片
     * @param logoName
     *            logo名称
     * @param format
     *            图片后缀名
     * @return
     */
    private byte[] addLogoQrCode(BufferedImage bim, byte[] logoPic, String logoName, String format) {
        try {
            /**
             * 读取二维码图片，并构建绘图对象
             */
            BufferedImage image = bim;
            Graphics2D g = image.createGraphics();
            if (StringUtils.isNotEmpty(logoName)) {
                g.setColor(Color.BLACK);
                // 字体、字型、字号
                g.setFont(new Font("宋体", Font.BOLD, 50));

                log.info("logoName1:"+logoName);
                int widthTemp = g.getFontMetrics().stringWidth(logoName);
                log.info("logoName2:"+logoName);
                // 画文字
                g.drawString(logoName, (WIDTH - widthTemp) / 2, HEIGHT - 60);
                image.flush();
            }

            if (logoPic != null) {
                /**
                 * 读取Logo图片
                 */
                try {
                    BufferedImage logo = ImageIO.read(new ByteArrayInputStream(logoPic));
                    /**
                     * 设置logo的大小,本人设置为二维码图片的20%,因为过大会盖掉二维码
                     */
                    int widthLogo = (int) Math.floor(logo.getWidth(null) > image.getWidth() * 1.5 / 10
                            ? (image.getWidth() * 1.5 / 10) : logo.getWidth(null));
                    int heightLogo = (int) Math.floor(logo.getHeight(null) > image.getHeight() * 1.5 / 10
                            ? (image.getHeight() * 1.5 / 10) : logo.getWidth(null));
                    /**
                     * logo放在中心
                     */
                    int x = (int) Math.floor((image.getWidth() - widthLogo) / 2);
                    int y = (int) Math.floor((image.getHeight() - heightLogo) / 2);
                    /**
                     * logo放在右下角
                     */
                    // 开始绘制图片
                    g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
                    g.drawImage(logo, x, y, widthLogo, heightLogo, null);
                    g.drawRect(x, y, widthLogo, heightLogo);
                    // logo边框大小
                    g.setStroke(new BasicStroke(2));
                    // logo边框颜色
                    g.setColor(Color.WHITE);
                    g.drawRect(x, y, widthLogo, heightLogo);
                    logo.flush();
                    image.flush();
                } catch (Exception e) {
                    log.error(e.getMessage());
                }
            }

            g.dispose();
            image.flush();
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            baos.flush();
            ImageIO.write(image, format, baos);
            return baos.toByteArray();
        } catch (Exception e) {
            log.error(e.getMessage());
        }
        return null;
    }

    /**
     * 生成二维码bufferedImage图片
     *
     * @param content
     *            编码内容
     * @param barcodeFormat
     *            编码类型
     * @param width
     *            图片宽度
     * @param height
     *            图片高度
     * @param hints
     *            设置参数
     * @return
     */
    private BufferedImage getQrCodeBufferedImage(String content, BarcodeFormat barcodeFormat, int width, int height,
                                                         Map<EncodeHintType, ?> hints) {
        MultiFormatWriter multiFormatWriter = null;
        BitMatrix bm = null;
        BufferedImage image = null;
        try {
            multiFormatWriter = new MultiFormatWriter();
            // 参数顺序分别为：编码内容，编码类型，生成图片宽度，生成图片高度，设置参数
            bm = multiFormatWriter.encode(content, barcodeFormat, width, height, hints);
            int w = bm.getWidth();
            int h = bm.getHeight();
            image = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);

            // 开始利用二维码数据创建Bitmap图片，分别设为黑（0xFFFFFFFF）白（0xFF000000）两色
            for (int x = 0; x < w; x++) {
                for (int y = 0; y < h; y++) {
                    image.setRGB(x, y, bm.get(x, y) ? QRCOLOR : BGWHITE);
                }
            }
        } catch (WriterException e) {
            log.warn(e);
        }
        return image;
    }
}
